"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = exports.PathResolver = void 0;
const path_to_regexp_1 = require("path-to-regexp");
const contract_1 = require("./contract");
const globals_1 = require("../globals");
class PathResolver {
    constructor(routes, options = contract_1.defaultOptions) {
        this.options = options;
        this.routes = [];
        routes.forEach(r => {
            const route = globals_1.cloneRoute(r);
            if (route.path[0] === '/')
                throw errorLeadSlash(route.path);
            // Step 1. All root paths get a prefix '/'
            route.path = exports.init.calcPath(route.path, '/');
            route.redirectTo = exports.init.calcTo(route.redirectTo, '/');
            route.customTo = exports.init.calcCustomTo(route.customTo, '/');
            // Step 2. All children paths are obtained as follows:
            //         [full parent path] + '/' + [child path]
            route.children = exports.init.children(route);
            this.routes.push(route);
        });
    }
    resolve(pathname) {
        this.trace(`resolving '${pathname}'`);
        return this.find(pathname, this.routes);
    }
    find(pathname, routes, parentRoute) {
        if (!routes)
            return;
        for (let i = 0; i < routes.length; i++) {
            const route = globals_1.cloneRoute(routes[i]);
            const matching = path_to_regexp_1.match(route.path)(pathname);
            this.trace(`[${!!matching ? 'v' : 'x'}] ${route.path}`);
            if (matching && !needToMatchChildren(route)) {
                const pathParams = matching.params;
                if (route.redirectTo) {
                    route.redirectTo = path_to_regexp_1.compile(route.redirectTo)(pathParams);
                }
                if (route.customTo) {
                    route.customTo.pathname = path_to_regexp_1.compile(route.customTo.pathname)(pathParams);
                }
                return { route, pathParams, parentRoute };
            }
            const found = this.find(pathname, route.children, route);
            if (found)
                return found;
        }
    }
    correctResultFromAction(pathname, result, route, parentRoute) {
        if (result.redirectTo === undefined && result.customTo === undefined)
            return;
        const res = globals_1.cloneResult(result);
        const parentPath = parentRoute ? parentRoute.path : '/';
        result.redirectTo = exports.init.calcTo(result.redirectTo, parentPath);
        result.customTo = exports.init.calcCustomTo(result.customTo, parentPath);
        const matching = path_to_regexp_1.match(route.path)(pathname);
        if (matching) {
            const pathParams = matching.params;
            if (result.redirectTo !== undefined) {
                result.redirectTo = path_to_regexp_1.compile(result.redirectTo)(pathParams);
            }
            if (result.customTo) {
                result.customTo.pathname = path_to_regexp_1.compile(result.customTo.pathname)(pathParams);
            }
        }
    }
    trace(path) {
        if (this.options.enableTrace)
            console.log(' ', path);
    }
}
exports.PathResolver = PathResolver;
exports.init = {
    calcPath: (path, parentPath) => {
        if (path === '') {
            return parentPath;
        }
        else {
            const prefix = parentPath === '/' ? '' : parentPath;
            return prefix + '/' + path;
        }
    },
    calcTo: (to, parentPath) => {
        if (typeof to === 'string') {
            if (to === '')
                return parentPath;
            else if (to[0] === '/')
                return to;
            else {
                const prefix = parentPath === '/' ? '/' : parentPath + '/';
                return prefix + to;
            }
        }
    },
    calcCustomTo: function (to, parentPath) {
        if (to) {
            to.pathname = this.calcTo(to.pathname, parentPath);
            return to;
        }
    },
    children: function ({ path: parentPath, children: routes }) {
        if (!routes)
            return;
        const children = [];
        for (let i = 0; i < routes.length; i++) {
            const route = globals_1.cloneRoute(routes[i]);
            const { path } = route;
            if (path[0] === '/')
                throw errorLeadSlash(path);
            children.push(route);
            route.path = this.calcPath(path, parentPath);
            route.redirectTo = this.calcTo(route.redirectTo, parentPath);
            route.customTo = this.calcCustomTo(route.customTo, parentPath);
            route.children = this.children(route);
        }
        return children;
    },
};
const needToMatchChildren = ({ redirectTo, component, children }) => redirectTo === undefined && component === undefined // if 'redirectTo' and 'component' are omitted
    && children !== undefined && children.length > 0 // but children are available
;
const errorLeadSlash = (path) => new Error(`Invalid configuration of route, because path [ ${path} ] cannot start with a slash`);
